<?php

namespace Ems\Services;

use Ems\Services\{
    AdmissaoService,
    EnderecoService,
    PessoaService
};
use Ems\Repository\GeralRepository;
use Ems\Util\Utils;
use Util\DateUtil;
use Util\Environment;
use Doctrine\ORM\EntityManager;
class GupyService {
    public const ORG_20_TOKEN = '50d22558-0c0f-4b62-97e2-e459e886a7c1';
    public const DEFAULT_ORG = 20;
    public const DEFAULT_OPERADOR = '000000000001';

    /** @var EnderecoService */
    protected $enderecoService;

    /** @var GeralRepository */
    protected $geralRepository;

    /** @var Environment */
    protected $environment;

    /** @var PessoaService */
    protected $pessoaService;

    /** @var AdmissaoService */
    protected $admissaoService;

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(
        GeralRepository $geralRepository,
        EnderecoService $enderecoService,
        Environment $environment,
        PessoaService $pessoaService,
        EntityManager $entityManager
    ) {
        $this->geralRepository = $geralRepository;
        $this->enderecoService = $enderecoService;
        $this->environment = $environment;
        $this->pessoaService = $pessoaService;
        $this->entityManager = $entityManager;
    }

    public function gupyPreEmployeeMoved(
        array $row
    ) {
        $this->verifyAuthentication();

        $this->environment->setNrOrg(self::DEFAULT_ORG);
        $this->environment->setCdOperador(self::DEFAULT_OPERADOR);

        $data = $row['data'];
        $this->verifyData($data);

        $candidate = $data['candidate'] ?? null;
        $documents = $data['admission']['documents']['preEmployee'] ?? null;
        $dependentsDocuments = $data['admission']['documents']['dependents'] ?? [];
        $dtAdmissaoPreliminar = $data['admission']['hiringDate'] ?? null;

        $this->admissaoService = Utils::getInstance('AdmissaoService');

        $admissaoRow = $this->createAdmissaoRow($candidate, $documents, $dtAdmissaoPreliminar);
        $nmMae = $admissaoRow['dadosPessoais']['NMMAE'] ?? null;
        $nmPai = $admissaoRow['dadosPessoais']['NMPAI'] ?? null;

        $this->entityManager->beginTransaction();

        try {
            [$nrParcNegocio, $nrPessoa, $nrPessoaH] = $this->admissaoService->setAdmissaoSimplificada($admissaoRow);
            $this->setDependentes($dependentsDocuments, $nrParcNegocio, $nmMae, $nmPai);

            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    private function verifyData(
        array $data
    ) {
        $candidate = $data['candidate'] ?? null;
        $status = $data['admission']['status'] ?? null;
        $documents = $data['admission']['documents']['preEmployee'] ?? null;
        $cpf = $candidate['identificationDocument'] ?? null;

        $validStati = [
            'SEND_DOCUMENTS',
            'SIGNING_CONTRACT',
        ];

        // Não trocar var_dump por Exception!!!
        if (!$candidate) {
            var_dump('Candidato não informado!'); die;
        }

        if (!in_array($status, $validStati)) {
            var_dump('Status inválido'); die;
        }

        if (!$cpf) {
            var_dump('CPF não informado!'); die;
        }

        if (!$documents) {
            var_dump('Não há documentos!'); die;
        }

        if (!$this->documentsHaveValues($documents)) {
            var_dump('Documentos não têm resposta!'); die;
        }

        if ($this->candidateIsRegistered($candidate)) {
            var_dump('O candidato já está cadastrado!'); die;
        }
    }

    private function candidateIsRegistered(
        array $candidate
    ): bool {
        $cpf = $candidate['identificationDocument'];
        $pessoa = $this->pessoaService->getPessoaByCpf($cpf);
        $isRegistered = $pessoa !== false;

        return $isRegistered;
    }

    private function documentsHaveValues(
        array $documents
    ): bool {
        $firstValue = $this->getDocumentValues($documents, 3);

        return !!$firstValue;
    }

    private function verifyAuthentication() {
        $headers = getallheaders();
        $authenticationHeader =
            $headers['Authorization']
            ?? $headers['authorization']
            ?? $headers['AUTHORIZATION']
            ?? null;

        if ($authenticationHeader === null) {
            var_dump('Token de autenticação não está presente!', $headers);die;
        }

        $validToken = strpos($authenticationHeader, self::ORG_20_TOKEN) !== false;

        if (!$validToken) {
            var_dump('Invalid Gupy Token!');die;
        }
    }

    private function createAdmissaoRow(
        array $candidate,
        array $documents,
        $dtAdmissaoPreliminar
    ): array {
        $dadosPessoais = $this->getDadosPessoaisRow($candidate, $documents, $dtAdmissaoPreliminar);
        $comunicacao = $this->getComunicacaoRow($candidate);
        $endereco = $this->getEnderecoRow($documents);
        $conta = $this->getContaRow($documents);
        $documentos = [];

        $admissaoRow = [
            'CDOPERADOR' => self::DEFAULT_OPERADOR,
            'dadosPessoais' => $dadosPessoais,
            'comunicacao' => $comunicacao,
            'endereco' => $endereco,
            'conta' => $conta,
            'documentos' => $documentos,
        ];

        return $admissaoRow;
    }

    private function getDocumentValues(
        array $documents,
        int $docId
    ) {
        foreach ($documents as $doc) {
            if ($doc['id'] === $docId) {
                $docValues = $doc['values'] ?? null;
                return $docValues;
            }
        }

        return null;
    }

    private function getDadosPessoaisRow(
        array $candidate,
        array $documents,
        $dtAdmissaoPreliminar
    ): array {
        $rgValues = $this->getDocumentValues($documents, 3) ?? [];
        $pisValues = $this->getDocumentValues($documents, 40035) ?? [];
        $escolaridadeValues = $this->getDocumentValues($documents, 40134) ?? [];
        $condicaoFisValues = $this->getDocumentValues($documents, 40121) ?? [];
        $militarValues = $this->getDocumentValues($documents, 40133) ?? [];

        $candidateRow = $this->getDataFromCandidate($candidate);
        $rgRow = $this->getDataFromRG($rgValues);
        $pisRow = $this->getDataFromPis($pisValues);
        $escolaridadeRow = $this->getDataFromComprovanteEscolar($escolaridadeValues);
        $condicaoFisRow = $this->getDataFromCondicaoFisica($condicaoFisValues);
        $militarRow = $this->getDataFromCertReservista($militarValues);

        $dadosPessoais = array_merge(
            $candidateRow,
            $rgRow,
            $pisRow,
            $escolaridadeRow,
            $condicaoFisRow,
            $militarRow
        );

        // caso precise definir a data de admissão preliminar na row, basta
        // descomentar a linha abaixo
        // $dadosPessoais['DTADMISSAOPRE'] = $dtAdmissaoPreliminar;
        $dadosPessoais['DTMESCOMPETENC'] = date('01/m/Y');

        return $dadosPessoais;
    }

    private function getDataFromCandidate(
        array $candidate
    ): array {
        $cpf = $candidate['identificationDocument'] ?? null;

        $disabilities = $candidate['disabilities'] ?? null;
        $idCotaPCD = $disabilities ? 'S' : 'N';

        $row = [
            'NRCPFPESSOA' => $cpf,
            'IDCOTAPCD' => $idCotaPCD,
        ];

        return $row;
    }

    private function getEnderecoRow(
        array $documents
    ): array {
        $enderecoValues = $this->getDocumentValues($documents, 2) ?? [];
        $enderecoRow = $this->getDataFromEndereco($enderecoValues);

        return $enderecoRow;
    }

    private function getContaRow(
        array $documents
    ): array {
        $contaValues = $this->getDocumentValues($documents, 40032) ?? [];
        $contaRow = $this->getDataFromDadosBancarios($contaValues);

        return $contaRow;
    }

    private function getComunicacaoRow(
        array $candidate
    ): array {
        $telefone = empty($candidate['phoneNumber']) ?
            null :
            $candidate['phoneNumber'];

        [$ddiTel, $dddTel, $phoneTel] = $this->getPhoneInfo($telefone);

        $celular = empty($candidate['mobileNumber']) ?
            null :
            $candidate['mobileNumber'];

        [$ddiCel, $dddCel, $phoneCel] = $this->getPhoneInfo($celular);

        $email = $candidate['email'];

        $comunicacao = [
            'TELPRINCIPAL' => $phoneTel,
            'PREFTELPRINCIPAL' => $dddTel,
            'CELPARTIC' => $phoneCel,
            'PREFCELPARTIC' => $dddCel,
            'EMAIL' => $email,
        ];

        return $comunicacao;
    }

    // Obs: funciona apenas para números de
    // telefone no formato brasileiro
    private function getPhoneInfo(
        $phoneNumber
    ): array {
        if (!$phoneNumber) {
            return [null, null, null];
        }

        $ddi = substr($phoneNumber, 0, 3);
        $ddd = substr($phoneNumber, 3, 2);
        $phone = substr($phoneNumber, 5);

        $phoneInfo = [$ddi, $ddd, $phone];
        return $phoneInfo;
    }

    private function getDataFromEndereco(
        array $docValues
     ): array {
        $cdPais = null;
        $sgEstado = null;
        $cdMunicipio = null;
        $cdBairro = null;
        $cdLogradouro = null;

        $nmPais         = $docValues['pais'] ?? null;
        $nmEstado       = $docValues['estado'] ?? null;
        $nmMunicipio    = $docValues['cidade'] ?? null;
        $cep            = $docValues['cep'] ?? null;
        $nmBairro       = $docValues['bairro'] ?? null;
        $endereco       = $docValues['logradouro'] ?? null;
        $numero         = $docValues['numero'] ?? null;
        $complemento    = $docValues['complemento'] ?? null;

        if ($nmPais) {
            $pais = $this->geralRepository->getPaisByNameOrSg($nmPais);

            if ($pais) {
                $cdPais = $pais['CDPAIS'];
            }
        }

        if ($cdPais && $nmEstado) {
            $estado = $this->geralRepository->getEstadoByNameOrSg(
                $cdPais,
                $nmEstado
            );

            if ($estado) {
                $sgEstado = $estado['SGESTADO'];
            }
        }

        if ($cdPais && $sgEstado && $nmMunicipio) {
            $municipio = $this->geralRepository->getCidadeByName(
                $cdPais,
                $sgEstado,
                $nmMunicipio
            );

            if ($municipio) {
                $cdMunicipio = $municipio['CDMUNICIPIO'];
            }
        }

        if ($cdPais && $sgEstado && $cdMunicipio && $nmBairro) {
            $bairro = $this->geralRepository->getBairroByName(
                $cdPais,
                $sgEstado,
                $cdMunicipio,
                $nmBairro
            );

            if ($bairro) {
                $cdBairro = $bairro['CDBAIRRO'];
            }
        }

        if ($endereco) {
            $logradouro = explode(' ', $endereco)[0] ?? null;

            if ($logradouro) {
                $cdLogradouro = $this->enderecoService->getCdLogradouroByName($logradouro);
            }
        }

        $row = [
            'CDTIPOENDERECO' => 'PRINCIPAL',
            'CDPAIS' => $cdPais,
            'SGESTADO' => $sgEstado,
            'CDMUNICIPIO' => $cdMunicipio,
            'NRCEPENDERECO' => $cep,
            'NMBAIRROENDERECO' => $nmBairro,
            'CDBAIRRO' => $cdBairro,
            'CDLOGRADOURO' => $cdLogradouro,
            'DSENDERECO' => $endereco,
            'NRIMOVELENDERECO' => $numero,
            'DSCOMPLEENDERECO' => $complemento,
        ];

        return $row;
    }

    private function getDataFromRG(
        array $docValues
     ): array {
        $cdEstadoCivil = null;
        $nrRaca = null;
        $nrNacionalidade = null;
        $cdPais = null;
        $sgEstado = null;
        $cdMunicipio = null;

        $rg = $docValues['numero-do-rg'] ?? null;
        $dtExpedicaoRg = $docValues['data-de-expedicao'] ?? null;
        $ufEmissor = $docValues['uf-emissor'] ?? null;
        // $orgaoEmissor = $docValues['orgao-emissor'] ?? null;

        $nmEstadoCivil = $docValues['estado-civil'] ?? null;

        $nome = $docValues['nome'] ?? null;
        $dtNacimento = $docValues['data-de-nascimento'] ?? null;
        $genero = $docValues['genero'] ?? null;
        $etnia = $docValues['etnia'] ?? null;
        $idSexo = $genero === 'Masculino' ? 'M' : 'F';

        $nomeMae = $docValues['nome-da-mae'] ?? null;
        $nomePai = $docValues['nome-do-pai'] ?? null;

        $nmPais = $docValues['nacionalidade'] ?? null;
        $nmEstado = $docValues['estado-onde-nasceu'] ?? null;
        $nmMunicipio = $docValues['cidade-onde-nasceu'] ?? null;


        if ($nmEstadoCivil) {
            $cdEstadoCivil = $this->pessoaService->getCdEstadoCivilByNm(
                $nmEstadoCivil
            );
        }

        if ($etnia) {
            $nrRaca = $this->pessoaService->getNrRacaByNm(
                $etnia
            );
        }

        if ($nmPais) {
            $pais = $this->geralRepository->getPaisByNameOrSg(
                $nmPais
            );

            $cdPais = $pais['CDPAIS'] ?? null;

            if ($cdPais) {
                $nacionalidade = $this->geralRepository->getNacionalidadeByCdPais(
                    $cdPais
                );

                if ($nacionalidade) {
                    $nrNacionalidade = $nacionalidade['NRNACIONALIDADE'];
                }
            }
        }

        if ($cdPais && $nmEstado) {
            $estado = $this->geralRepository->getEstadoByNameOrSg(
                $cdPais,
                $nmEstado
            );

            if ($pais) {
                $sgEstado = $estado['SGESTADO'];
            }
        }

        if ($cdPais && $sgEstado && $nmMunicipio) {
            $municipio = $this->geralRepository->getCidadeByName(
                $cdPais,
                $sgEstado,
                $nmMunicipio
            );

            if ($municipio) {
                $cdMunicipio = $municipio['CDMUNICIPIO'];
            }
        }

        if ($dtExpedicaoRg) {
            $dtExpedicaoRg = DateUtil::iso8601ToDDMMYYYY($dtExpedicaoRg);
        }

        if ($dtNacimento) {
            $dtNacimento = DateUtil::iso8601ToDDMMYYYY($dtNacimento);
        }

        $row = [
            'NMMAE' => $nomeMae,
            'NMPAI' => $nomePai,
            'NMPESSOA' => $nome,
            'NRRGPESSOA' => $rg,
            'DTEXRGPESSOA' => $dtExpedicaoRg,
            'SGUFRGPESSOA' => $ufEmissor,
            'NRRACAPESSOA' => $nrRaca,
            'CDESTACIVIL' => $cdEstadoCivil,
            'NRNACIONALID' => $nrNacionalidade,
            'DTNASCPESSOA' => $dtNacimento,
            'IDSEXOPESSOA' => $idSexo,
            'CDPAIS' => $cdPais,
            'SGESTADO' => $sgEstado,
            'CDMUNICIPIO' => $cdMunicipio,
        ];

        return $row;
    }

    private function getDataFromPis(
        array $docValues
     ): array {
        $pis = $docValues['2a368cec-f982-4549-883e-d4771d0a2bc1'] ?? null;
        $dtPis = $docValues['8a0c7600-6bc9-4eab-bf1a-c855318a1a26'] ?? null;

        if ($dtPis) {
            $dtPis = DateUtil::iso8601ToDDMMYYYY($dtPis);
        }

        $row = [
            'NRPISPASEPPES' => $pis,
            'DTINSCPISPASEP' => $dtPis,
        ];

        return $row;
    }

    private function getDataFromComprovanteEscolar(
        array $docValues
     ): array {
        $escolaridade = $docValues['32f6ed00-e429-40b6-a1d0-7e18f0fe0059'] ?? null;

        $escolaridadeMap = [
            'Ensino Fundamental Completo' => 5,
            'Ensino Médio Cursando' => 6,
            'Ensino Médio Concluido' => 7,
            'Ensino Superior Cursando' => 8,
            'Ensino Superior Concluido' => 9,
        ];

        $nrGrauInstrucao = $escolaridadeMap[$escolaridade] ?? null;

        $row = [
            'NRGRAUINSTR' => $nrGrauInstrucao,
        ];

        return $row;
    }

    private function getDataFromDadosBancarios(
        array $docValues
     ): array {
        $banco = '033'; // Santander
        $agencia = $docValues['c02eae32-75c3-4198-bc60-35bc99cae383'] ?? null;
        $conta = $docValues['eaf131d7-98f8-4ac0-83a8-f8c6ff7f5be9'] ?? null;
        $tipoConta = 3; // Conta Salário

        $row = [
            'CDBANCO' => $banco,
            'CDAGENCIA' => $agencia,
            'CDCONTCORR' => $conta,
            'NRTPCONTCORR' => $tipoConta,
        ];

        return $row;
    }

    private function getDataFromVR(
        array $docValues
    ): array {
        return [];
    }

    private function getDataFromVT(
        array $docValues
    ): array {
        return [];
    }

    private function getDataFromCondicaoFisica(
        array $docValues
     ): array {
        $condicaoFisica = $docValues['ba187938-1ef7-41da-8c74-292f3331061c'] ?? null;

        $condicaoMap = [
            'Normal' => 1,
            'Deficiência Física' => 2,
            'Deficiência Visual' => 4,
            'Deficiência Mental' => 5,
            'Deficiência Auditiva' => 3,
            'Deficiência Múltipla' => 6,
            'Reabilitado' => 7,
        ];

        $nrCondicao = $condicaoMap[$condicaoFisica] ?? null;

        $row = [
            'NRCONDFISPES' => $nrCondicao,
        ];

        return $row;
    }

    private function getDataFromCertReservista(
        array $docValues
     ): array {
        $nrCertReservista = $docValues['30ed4c3a-ac60-443d-acb0-004eaf11a2fd'] ?? null;
        $dtEmissaoCert = $docValues['ea0b748e-161b-4091-8d5d-54a8f67434a2'] ?? null;

        if ($dtEmissaoCert) {
            $dtEmissaoCert = DateUtil::iso8601ToDDMMYYYY($dtEmissaoCert);
        }

        $row = [
            'NRCERTRESEPES' => $nrCertReservista,
            'DTCERTRESEPES' => $dtEmissaoCert,
        ];

        return $row;
    }

    private function setDependentes(
        array $dependentsDocuments,
        int $nrParcNegocio,
        $nmMae = null,
        $nmPai = null
    ) {
        $cdOperador = $this->environment->getCdOperador();
        $dependentes = [];
        $dtMesCompetencia = date('01/m/Y');

        if ($nmMae) {
            $rowMae = [
                'NMPESSOA' => $nmMae,
                'NRTIPORELACIONA' => 1, // mae
                'NRCPFPESSOA' => null,
                'DTNASCPESSOA' => null,
            ];

            $dependentes[] = $rowMae;
        }

        if ($nmPai) {
            $rowPai = [
                'NMPESSOA' => $nmPai,
                'NRTIPORELACIONA' => 2, // pai
                'NRCPFPESSOA' => null,
                'DTNASCPESSOA' => null,
            ];

            $dependentes[] = $rowPai;
        }

        foreach ($dependentsDocuments as $depDoc) {
            $dependenteRow = $this->getDependentRow($depDoc);

            if ($dependenteRow) {
                $nomeDep = $dependenteRow['NMPESSOA'];
                $nomeIgualMae = strcasecmp($nmMae, $nomeDep) === 0;
                $nomeIgualPai = strcasecmp($nmPai, $nomeDep) === 0;

                if (!$nomeIgualMae && !$nomeIgualPai) {
                    $dependentes[] = $dependenteRow;
                }
            }
        }

        $this->admissaoService->setDependentesAdmissao(
            $dependentes,
            $cdOperador,
            $nrParcNegocio,
            null,
            $dtMesCompetencia
        );
    }

    private function getDependentRow(
        array $depDoc
    ) {
        $documents = $depDoc['documents'] ?? null;
        $nrTipoRelacionamento = 3; // filho

        if (!$documents) {
            return false;
        }

        $dados = $this->getDependentDocumentValues($documents, 'a3c51d53-3c86-4142-aea8-ed9fc84e0e94');

        if (!$dados) {
            return false;
        }

        $dtNascimento   = $dados['birthDate'] ?? null;
        $cpf            = $dados['cpf'] ?? null;
        $nome           = $dados['name'] ?? null;

        if (!$dtNascimento || !$cpf || !$nome) {
            return false;
        }

        $row = [
            'NMPESSOA' => $nome,
            'NRCPFPESSOA' => $cpf,
            'DTNASCPESSOA' => $dtNascimento,
            'NRTIPORELACIONA' => $nrTipoRelacionamento,
        ];

        return $row;
    }

    private function getDependentDocumentValues(
        array $documents,
        string $docId
    ) {
        foreach ($documents as $doc) {
            if ($doc['id'] === $docId) {
                $docValues = $doc['values'] ?? null;
                return $docValues;
            }
        }

        return null;
    }
}